#!/usr/bin/python3
# Copyright 2016-2018 Francisco Pina Martins <f.pinamartins@gmail.com>
# This file is part of pyRona.
# pyRona is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# pyRona is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with pyRona.  If not, see <http://www.gnu.org/licenses/>.

import numpy as np


def mahalanobis_dist_calculator(x_coords, y_coords):
    """
    Calculates Mahalanobis Distance.
    Takes 2 np.array([]) as input which are used to calculate the MD distance.
    Returns a list with the MD distances between every xy point.
    http://kldavenport.com/mahalanobis-distance-and-outliers/
    """
    covariance_xy = np.cov(x_coords, y_coords, rowvar=0)
    inv_covariance_xy = np.linalg.inv(covariance_xy)
    xy_mean = np.mean(x_coords), np.mean(y_coords)
    x_diff = np.array([x_i - xy_mean[0] for x_i in x_coords])
    y_diff = np.array([y_i - xy_mean[1] for y_i in y_coords])
    diff_xy = np.transpose([x_diff, y_diff])

    mh_dist = []
    for _, item in enumerate(diff_xy):
        mh_dist.append(np.sqrt(np.dot(np.dot(np.transpose(item),
                                             inv_covariance_xy), item)))

    return mh_dist


def md_remove_outliers(x_coords, y_coords):
    """
    Removes outliers based on Mahalanobis Distance.
    Takes 2 np.array([]) as input which are used to calculate the MD distance.
    Returns an np.array([]) with the indices of the removed outliers.
    http://kldavenport.com/mahalanobis-distance-and-outliers/
    """
    mahalanobis_dists = mahalanobis_dist_calculator(x_coords, y_coords)
    threshold = np.mean(mahalanobis_dists) * 1.5  # adjust 1.5 accordingly

    # Single or no outliers approach
    # if outliers == 1:
    if max(mahalanobis_dists) >= threshold:
        outlier_indeces = [mahalanobis_dists.index(max(mahalanobis_dists))]
    else:
        outlier_indeces = []

    # # Multiple outlier approach
    # elif outliers == 2:
    #     n_x, n_y, outlier_indeces = [], [], []
    #     for i, item in enumerate(mahalanobis_dists):
    #         if item <= threshold:
    #             n_x.append(x_coords[i])
    #             n_y.append(y_coords[i])
    #         else:
    #             outlier_indeces.append(i)  # position of removed pair

    return np.array(outlier_indeces)



#!/usr/bin/python3
# Copyright 2016-2018 Francisco Pina Martins <f.pinamartins@gmail.com>
# This file is part of pyRona.
# pyRona is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# pyRona is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with pyRona.  If not, see <http://www.gnu.org/licenses/>.


from collections import defaultdict
from collections import OrderedDict
import numpy as np


# Baypass exclusive functions
def parse_baypass_envfile(envfile_filename):
    """
    Parses an ENVFILE (baypass) input file and returns a list of np arrays
    (one per line)
    """
    envfile = open(envfile_filename, 'r')
    covariates = []
    for lines in envfile:
        covariates.append(np.array([float(x) for x in lines.split()]))

    envfile.close()

    return covariates


def popnames_parser(popnames_file):
    """
    Parses a file with population names and returns a list with these names.
    The order must be the same as in the baypass input files (geno and env).
    """
    popnames = []

    popfile = open(popnames_file, 'r')
    for lines in popfile:
        popnames.append(lines.strip())

    popfile.close()

    return popnames


def baypass_summary_betai_parser(summary_filename, bf_treshold, immutables):
    """
    Parses a baypass summary file to extract any significant associations
    between a marker and a covariate.
    Returns a list of associations tuples:
    [('marker', 'covariate'), ('marker', 'covariate')...]
    """
    summary = open(summary_filename, 'r')
    header = summary.readline()  # Skip header and get BF column.
    for identifier in ("eBPmc", "BF(dB)"):
        try:
            bf_col = header.strip().split().index(identifier)
        except ValueError:
            pass
    associations = []
    for lines in summary:
        splitline = lines.strip().split()
        covariate = splitline[0]
        # Remove "immutable" covariates (Lat, Long, Alt)
        if covariate not in immutables:
            marker_id = splitline[1]
            bf_value = float(splitline[bf_col])
            if bf_value >= float(bf_treshold):
                associations.append((marker_id, covariate))

    summary.close()

    return associations


def baypass_pij_parser(pij_filename, associations):
    """
    Parses a baypass pij file to extract standardized allelic frequencies.
    Returns a dict with markers as keys and a np.array of allelic frequencies
    as values:
    {marker:np.array([freq_pop1, freq_pop2, ...])}
    """
    marker_list = [x for x, y in associations]
    frequencies = defaultdict(list)

    pij = open(pij_filename, 'r')
    for lines in pij:
        splitline = lines.strip().split()
        if splitline[1] in marker_list:
            frequencies[splitline[1]].append(float(splitline[4]))

    pij.close()

    # Make sure we have np.arrays for the freqencies
    for key in frequencies.keys():
        frequencies[key] = np.array(frequencies[key])

    return frequencies


# LFMM exclusive functions
def lfmm_results_parser(lfmm_results_filename, assoc_threshold, immutables):
    """
    Parses a lfmm results file to extract any significant associations
    between a marker and a covariate.
    Returns a list of associations tuples:
    [('marker', 'covariate'), ('marker', 'covariate')...]
    """
    associations = []
    snp = 0
    imut_indeces = [int(x) for x in immutables]
    results = open(lfmm_results_filename, "r")
    for lines in results:
        snp += 1
        lines = [float(x) for x in lines.split(",")]
        try:
            snp_assocs = [(str(snp), str(i)) for i in line_index
                          if lines[i] < assoc_threshold]
        except NameError:
            line_index = list(range(len(lines)))
            for index in sorted(imut_indeces, reverse=True):
                del line_index[index]
            snp_assocs = [(str(snp), str(i)) for i in line_index
                          if lines[i] < assoc_threshold]

        associations += snp_assocs

    results.close()

    return associations


def lfmm_to_pop_allele_freqs(lfmm_filename, env_filename, associations,
                             popnames=False):
    """
    Parses a LFMM input file and an evironemtal variables file to group
    the allele frequencies into "populations".
    The enviromental file should have the population names in the first
    column.
    Returns a dict with markers as keys and a np.array of allelic frequencies
    as values:
    {marker:np.array([freq_pop1, freq_pop2, ...])}
    """
    def _process_alleles(allele_matrix, indices):
        """
        Turns allele counts int allele frequencies.
        """
        frequencies = []
        allele_matrix = list(allele_matrix)

        absolutes = []
        startindex = 0
        for i in indices:
            alleles = allele_matrix[startindex:i + 1]
            absolutes.append(alleles)
            al_count = len(alleles)
            total = (al_count - alleles.count(9)) * 2
            try:
                freq = sum([0 if x == 9 else x for x in alleles]) / total
            except ZeroDivisionError:
                freq = np.nan

            frequencies.append(freq)
            startindex = i + 1

        return frequencies

    env_vars = open(env_filename, "r")
    pops = []
    indices = []
    for lines in env_vars:
        popname = lines.split()[0]
        try:
            if popname != pops[-1]:
                indices.append(len(pops) - 1)
        except IndexError:
            pass
        pops.append(popname)
    indices.append(len(pops) - 1)

    env_vars.close()

    collapsed_pops = list(OrderedDict.fromkeys(pops))

    id_freqs = {}
    lfmm = np.genfromtxt(lfmm_filename, delimiter=None, dtype=float)
    snp_num = 0

    markers, _ = zip(*associations)
    markers = set(markers)

    for snp in lfmm.T:
        snp_num += 1
        if str(snp_num) in markers:
            snp_data = _process_alleles(snp, indices)
            if snp_data is None:
                id_freqs[str(snp_num)] = None
            else:
                id_freqs[str(snp_num)] = np.array(snp_data)

    if popnames:
        return collapsed_pops, id_freqs
    else:
        return id_freqs


def parse_lfmm_envfile(envfile_filename):
    """
    Parses an ENVFILE (lfmm) input file and returns a list of np arrays
    (one per line). Covariate values are averaged per "population".
    """
    envfile = open(envfile_filename, 'r')

    env_data = {}
    for lines in envfile:
        lines = lines.strip().split()
        if lines[0] not in env_data:
            env_data[lines[0]] = np.array([float(x) for x in lines[1:]])
        else:
            env_data[lines[0]] = np.vstack((env_data[lines[0]],
                                            np.array([float(x) for x in
                                                      lines[1:]])))

    envfile.close()

    covariates = []
    for pop in env_data.values():
        covariates.append(np.array([np.average(x) for x in pop.T]))
    covariates = np.array(covariates)
    return list(covariates.T)



#!/usr/bin/python3
# Copyright 2016-2018 Francisco Pina Martins <f.pinamartins@gmail.com>
# This file is part of pyRona.
# pyRona is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# pyRona is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with pyRona.  If not, see <http://www.gnu.org/licenses/>.


from collections import defaultdict
from sys import argv

import numpy as np
try:
    import md_outlier_remover as mor
    import plotters.general_plotter as gp
    import file_parser as fp
    from argparser import argument_parser
except ImportError:
    import pyRona.md_outlier_remover as mor
    import pyRona.plotters.general_plotter as gp
    import pyRona.file_parser as fp
    from pyRona.argparser import argument_parser


class RonaClass:
    """
    Stores the RONA values for each covar
    """
    POP_NAMES = []

    def __init__(self, covar):
        self.name = covar
        self.pop_ronas = defaultdict(list)
        self.corr_coef = {}

        self.avg_ronas = []
        self.stderr_ronas = []

    def basic_stats(self, use_weights):
        """
        Gets the average RONA and stdev per population for each associated
        covariate. Stores the values in variables inside the class instance.
        """
        if len(self.pop_ronas) > 1:
            # Sort markers:
            markers = sorted(list(self.corr_coef.keys()))

            list_of_marker_values = np.array([self.pop_ronas[x] for x in
                                              markers],
                                             dtype=float)
            corr_weights = np.array([self.corr_coef[x] for x in markers],
                                    dtype=float)

            for i in list_of_marker_values.T:
                not_nans = ~np.isnan(i)
                if use_weights is True:
                    if True in not_nans:
                        self.avg_ronas += [np.average(i[not_nans],
                                                      weights=corr_weights
                                                      [not_nans])]
                    else:
                        self.avg_ronas += [np.nan]
                else:
                    self.avg_ronas += [np.average(i[not_nans])]

                self.stderr_ronas += [np.std(i[not_nans])
                                      / np.sqrt(len(i[not_nans]))]
        else:
            self.avg_ronas = [x for x in self.pop_ronas.values()][0]
            self.stderr_ronas = [0.0] * len(list(self.pop_ronas.values())[0])

    def count_markers(self):
        """
        Counts the number of markers in the instance.
        """
        return len(self.pop_ronas)


def calculate_rona(marker_name, rona, present_covar, future_covar,
                   allele_freqs, plot, outliers, rtype):
    """
    Calculates the "Risk of non adaptation" (RONA) of each popuation for a
    given association.
    Also plots the associations if requested.
    """
    # Remove outliers
    if outliers is True:
        outlier_pos = mor.md_remove_outliers(present_covar, allele_freqs)
        outlier_pos = np.array(outlier_pos, dtype='int')
        for i in outlier_pos:
            present_covar[i] = np.nan
            future_covar[i] = np.nan
            allele_freqs[i] = np.nan

        rona.pop_names = np.delete(RonaClass.POP_NAMES, outlier_pos)
    else:
        rona.pop_names = RonaClass.POP_NAMES

    # Calculate trendline:
    not_nan = ~np.isnan(present_covar)
    fit = np.polyfit(present_covar[not_nan], allele_freqs[not_nan], 1)
    fit_fn = np.poly1d(fit)

    # Get R²:
    rona.corr_coef[marker_name] = np.corrcoef(present_covar[not_nan],
                                              allele_freqs[not_nan])[1, 0] ** 2

    for pres, fut, freq in zip(present_covar, future_covar, allele_freqs):

        pres_distance = freq - fit_fn(pres)
        fut_distance = freq - fit_fn(fut)
        distance_diff = abs(pres_distance) - abs(fut_distance)

        amplitude = max(allele_freqs) - min(allele_freqs)

        if rtype == "diff":
            rel_distance = distance_diff / amplitude

        elif rtype == "absdiff":
            rel_distance = abs(distance_diff) / amplitude

        elif rtype == "dist":
            rel_distance = abs(fut_distance)

        rona.pop_ronas[marker_name] += [rel_distance]

    if plot is not None:
        gp.draw_individual_plots(present_covar, future_covar, rona,
                                 marker_name, allele_freqs, fit_fn, plot)


def results_summary(ronas, use_weights):
    """
    This function outputs a summary of the RONAS for each population and
    covariate.
    """
    pop_names = ronas[0].pop_names
    for i, j in enumerate(pop_names):
        if i == 0:
            print("Covar\t%s" % "\t".join([x.name for x in ronas]))
            print("#SNPs\t%s" % "\t".join([str(x.count_markers()) for x in
                                           ronas]))
        print("%s\t%s" % (j, "\t".join([str(x.avg_ronas[i]) for x in ronas])))

    print("Min R^2\t%s" %
          "\t".join([str(np.nanmin(list(x.corr_coef.values()))) for x in
                     ronas]))

    print("Max R^2\t%s" %
          "\t".join([str(np.nanmax(list(x.corr_coef.values()))) for x in
                     ronas]))

    # if use_weights is True:
    #     means = [str(np.average(list(x.corr_coef.values()),
    #                             weights=list(x.corr_coef.values()))) for x in
    #              ronas]
    # else:
    means = [str(np.nanmean(list(x.corr_coef.values()))) for x in ronas]
    print("Average R^2\t%s" % "\t".join(means))


def ronas_filterer(ronas, use_weights, num_covars):
    """
    Filters RONAS to remove immutable covars, and return only the top "n" most
    represented covariables.
    """
    sortable_representation = {}
    for k, rona in ronas.items():
        rona.basic_stats(use_weights)
        sortable_representation[k] = len(rona.pop_ronas)

    top_represented = sorted(sortable_representation,
                             key=sortable_representation.get,
                             reverse=True)[:num_covars]
    top_ronas = [ronas[x] for x in top_represented]

    return top_ronas


def main():
    """
    Main function. Takes all the inputs as arguments and runs the remaining
    functions of the program.
    """
    if len(argv) < 2:
        arg_list = ["-h"]
    else:
        arg_list = argv[1:]

    arg = argument_parser(arg_list)

    if arg.upstream == "baypass":
        present_covariates = fp.parse_baypass_envfile(arg.present_covars_file)
        future_covariates = fp.parse_baypass_envfile(arg.future_covars_file)
        RonaClass.POP_NAMES = fp.popnames_parser(arg.popnames_file)
        assocs = fp.baypass_summary_betai_parser(
            arg.baypass_summary_betai_file,
            arg.bayes_factor, arg.immutables)
        al_freqs = fp.baypass_pij_parser(arg.baypass_pij_file, assocs)
    elif arg.upstream == "lfmm":
        present_covariates = fp.parse_lfmm_envfile(arg.present_covars_file)
        future_covariates = fp.parse_lfmm_envfile(arg.future_covars_file)
        assocs = fp.lfmm_results_parser(arg.lfmm_assoc_file,
                                        arg.p_thres,
                                        arg.immutables)
        RonaClass.POP_NAMES, al_freqs = fp.lfmm_to_pop_allele_freqs(
            arg.allele_freqs_file,
            arg.present_covars_file,
            assocs,
            popnames=True)

    ronas = {}
    for assoc in assocs:
        marker, covar = assoc

        # Instanciate class
        if covar not in ronas:
            rona = RonaClass(covar)
        else:
            rona = ronas[covar]

        calculate_rona(marker, rona, present_covariates[int(covar) - 1],
                       future_covariates[int(covar) - 1],
                       al_freqs[marker],
                       arg.plots, arg.outliers, arg.rtype)

        ronas[covar] = rona

    ronas = ronas_filterer(ronas, arg.use_weights, arg.num_covars)

    results_summary(ronas, arg.use_weights)
    gp.draw_rona_plot(ronas, arg.outfile)

    if arg.map_filename is not None:
        # The map plotting module is only imported if a map plot is requested.
        # This is to be able to keep 'cartopy' as an optional dependency.
        try:
            import plotters.map_plotter as mapper
        except ImportError:
            import pyRona.plotters.map_plotter as mapper
        mapper.map_plotter(ronas, present_covariates[1], present_covariates[0],
                           arg.map_filename)


if __name__ == "__main__":
    main()



#!/usr/bin/python3
# Copyright 2018 Francisco Pina Martins <f.pinamartins@gmail.com>
# This file is part of pyRona.
# pyRona is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# pyRona is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with pyRona.  If not, see <http://www.gnu.org/licenses/>.

try:
    import cartopy.crs as ccrs
    import cartopy.feature as cfeature
    from cartopy.mpl.gridliner import LONGITUDE_FORMATTER, LATITUDE_FORMATTER
except ImportError:
    print("\nError importing 'Cartopy'. Please look at pyRona's manual"
          "(http://pyrona.readthedocs.io/en/latest/install/#installing-cartopy)"
          " for information on how to install it on your system. Map plotting"
          " was **not** performed.")
    quit()
import matplotlib.pyplot as plt
import numpy as np


def map_plotter(ronas, latitudes, longitudes, out_filename):
    """
    Plots a map with each sampling site coloured per its averag RONA value
    """
    def _define_map_edges(latitudes, longitudes, padding=0.10):
        """
        Defines the edges of the map to be drawn.
        Takes a list of latitudes and longitudes as input and returns the map
         edges
        """
        # Define padding for the map edges
        hpad = padding
        vpad = padding

        # Get map edges
        max_lon = np.max(longitudes)
        max_lon = max_lon + abs(max_lon * vpad)
        min_lon = np.min(longitudes)
        min_lon = min_lon - abs(min_lon * vpad)

        max_lat = np.max(latitudes)
        max_lat = max_lat + abs(max_lat * hpad)
        min_lat = np.min(latitudes)
        min_lat = min_lat - abs(min_lat * hpad)

        return([min_lat, max_lat, min_lon, max_lon])

    max_ronas = []
    pop_names = ronas[0].pop_names
    for i, _ in enumerate(pop_names):
        max_ronas += [max([x.avg_ronas[i] for x in ronas])]

    fig = plt.figure(figsize=(22, 12), facecolor="none")
    map_area = plt.axes(projection=ccrs.PlateCarree())

    map_edges = _define_map_edges(latitudes, longitudes)

    map_area.set_extent(map_edges)
    map_area.coastlines(resolution='50m')
    cfeature.BORDERS.scale = "50m"
    map_area.add_feature(cfeature.BORDERS)

    # Draw sampling sites
    dotplot = map_area.scatter(latitudes, longitudes, c=max_ronas, s=700,
                               vmin=0, vmax=max(max_ronas),
                               transform=ccrs.PlateCarree(),
                               cmap='autumn_r', zorder=2)

    # Label the locations
    for label, x, y in zip(pop_names, latitudes, longitudes):
        map_area.annotate(label.strip().replace("_", " "), xy=(x, y),
                          xytext=(0, -28), textcoords='offset points',
                          ha='center', va='bottom', fontsize=13,
                          bbox=dict(boxstyle='round,pad=0.1',
                                    fc='lightgrey', edgecolor="none"))

    # Control x and y ticks
    gridlines = map_area.gridlines(draw_labels=True)
    gridlines.xlines = False
    gridlines.ylines = False
    gridlines.ylabels_right = False
    gridlines.xlabels_top = False
    gridlines.xformatter = LONGITUDE_FORMATTER
    gridlines.yformatter = LATITUDE_FORMATTER
    gridlines.xlabel_style = {'size': 22}
    gridlines.ylabel_style = {'size': 22}

    # Control x and y labels
    map_area.text(-0.10, 0.55, 'Latitude', va='bottom', ha='center',
                  rotation='vertical', rotation_mode='anchor',
                  transform=map_area.transAxes, fontsize=28)
    map_area.text(0.5, -0.12, 'Longitude', va='bottom', ha='center',
                  rotation='horizontal', rotation_mode='anchor',
                  transform=map_area.transAxes, fontsize=28)

    # Sidebar settings
    sidebar = fig.colorbar(dotplot)
    sidebar.ax.tick_params(labelsize=20)
    sidebar.set_label(label='RONA', size=30, weight='bold')

    # Save the map
    fig.savefig(out_filename)

# TODO: Eventually make an interpolation



#!/usr/bin/python3
# Copyright 2016-2018 Francisco Pina Martins <f.pinamartins@gmail.com>
# This file is part of pyRona.
# pyRona is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# pyRona is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with pyRona.  If not, see <http://www.gnu.org/licenses/>.

from os import path
import numpy as np
import matplotlib
matplotlib.use("Agg")
import matplotlib.pyplot as plt


COLOR_LIST = ['red', 'green', 'blue', 'cyan', 'magenta', 'black', 'orange',
              'yellow']


def draw_individual_plots(present_covar, future_covar, rona, marker_name,
                          allele_freqs, fit_fn, outpath):
    """
    Draws an individual marker vs covar plot.
    """
    all_covars = np.append(present_covar, future_covar)

    fig, axes = plt.subplots()
    # Set-up the plot
    axes.set_xlabel("Covariate %s" % rona.name)
    axes.set_ylabel('Marker %s standardized allele freqs.' % marker_name)
    axes.set_title('Linear regression plot')

    axes.plot(present_covar, allele_freqs, 'bo')
    axes.plot(future_covar, allele_freqs, 'go')
    axes.plot(all_covars, fit_fn(all_covars), 'r--')

    axes.set_xlim(min(all_covars) - np.nanmean(present_covar) * 0.1,
                  max(all_covars) + np.nanmean(present_covar) * 0.1)
    axes.set_ylim(min(allele_freqs) - min(allele_freqs) * 0.1,
                  max(allele_freqs) + max(allele_freqs) * 0.1)

    # Annotation
    for label, x, y in zip(rona.pop_names, present_covar, allele_freqs):
        axes.annotate(label.strip(), xy=(x, y), xytext=(-9, 9),
                      textcoords='offset points', ha='right',
                      va='bottom', bbox=dict(boxstyle='round,pad=0.1',
                                             fc='yellow',
                                             alpha=0.3),
                      arrowprops=dict(arrowstyle='->',
                                      connectionstyle='arc3,rad=0'))

    # Set outpath
    filename = "Cov{}_Mrk{}.pdf".format(rona.name, marker_name)
    if path.isdir(outpath):
        full_outpath = path.join(outpath, filename)
    else:
        exit("ERROR: '-draw-ind-plots' option must point to a directory!")

    # Draw the plot
    fig.savefig(full_outpath)
    plt.close(fig)


def draw_rona_plot(ronas, outpath):
    """
    Draws a RONA plot of the Nth most represented covariates.
    Plots the RONA+/-Stderr for each of the populations.
    """
    fig, ax = plt.subplots()
    width = 1 / (len(ronas) + 1)

    counter = 0
    axes = []
    names = []
    for rona in ronas:
        ind = np.arange((len(rona.pop_names)))
        rects = ax.bar(ind + width * counter, rona.avg_ronas, width,
                       color=COLOR_LIST[counter], yerr=rona.stderr_ronas,
                       alpha=0.5, error_kw=dict(ecolor='gray'))
        counter += 1
        axes.append(rects)
        names.append(rona.name)

    # add some text for labels, title and axes ticks
    ax.set_ylabel('RONA')

    # Define title:
    if len(ronas) > 1:
        title_msg = ("Average RONA per population for the most represented %s "
                     "covariates" % len(ronas))
    else:
        title_msg = "RONA per population for the most represented covariate"

    ax.set_title(title_msg)
    ax.set_xticks(ind + width)
    ax.set_xticklabels(rona.pop_names, rotation=45, va="top", ha="right")

    box = ax.get_position()
    ax.set_position([box.x0, box.y0, box.width * 0.9, box.height])

    ax.legend(axes, names, loc='center left', bbox_to_anchor=(1, 0.5))

    # plt.tight_layout()
    plt.gcf().subplots_adjust(bottom=0.18)
    plt.gcf().subplots_adjust(right=0.85)

    fig.savefig(outpath)

    plt.close(fig)



#!/usr/bin/python3

# Copyright 2017-2020 Francisco Pina Martins <f.pinamartins@gmail.com>
# This file is part of pyRona.
# pyRona is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# pyRona is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with pyRona. If not, see <http://www.gnu.org/licenses/>.


import sys
from setuptools import setup


class NotSupportedException(BaseException):
    """
    Dummy class to work as an exception in case someone attempts to install
    pyRona using pip2.
    """
    pass


if sys.version_info.major < 3:
    raise NotSupportedException("Only Python 3.x Supported")


# Set some variables (PKGBUILD inspired)
VERSION = "0.4.3"
URL = "https://gitlab.com/StuntsPT/pyRona"


setup(
    name="pyRona",
    version=VERSION,
    packages=["pyRona",
              "pyRona.plotters"],
    install_requires=["numpy",
                      "matplotlib"],
    description=('A python implementation of "Risk of non Adaptedness"'
                 'method (with a bit of R too!)'),
    url=URL,
    download_url="{0}/archive/v{1}.tar.gz".format(URL, VERSION),
    author="Francisco Pina-Martins",
    author_email="f.pinamartins@gmail.com",
    license="GPL3",
    classifiers=["Intended Audience :: Science/Research",
                 "License :: OSI Approved :: GNU General Public License v3 ("
                 "GPLv3)",
                 "Natural Language :: English",
                 "Operating System :: POSIX :: Linux",
                 "Topic :: Scientific/Engineering :: Bio-Informatics",
                 "Programming Language :: Python :: 3 :: Only",
                 "Programming Language :: Python :: 3.4",
                 "Programming Language :: Python :: 3.5",
                 "Programming Language :: Python :: 3.6",
                 "Programming Language :: Python :: 3.7",
                 "Programming Language :: Python :: 3.8"],
    data_files=[("bin", ["pyRona/R/Baypass_workflow.R",
                         "pyRona/R/LFMM_workflow.R"])],
    entry_points={
        "console_scripts": [
            "pyRona = pyRona.pyRona:main",
        ]
    },
)



#!/usr/bin/python3
# Copyright 2017-2018 Francisco Pina Martins <f.pinamartins@gmail.com>
# This file is part of pyRona.
# pyRona is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# pyRona is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with pyRona.  If not, see <http://www.gnu.org/licenses/>.


import argparse as ap


def argument_parser(args):
    """
    Parses arguments and returns them in a neat variable.
    """
    parser = ap.ArgumentParser(description="A program to calculate "
                                           "the 'Risk of Non Adaptation' "
                                           "(RONA), based on BayPass "
                                           "output.",
                               prog="pyRona",
                               formatter_class=ap.HelpFormatter)

    subparsers = parser.add_subparsers(dest='upstream')
    baypass_parser = subparsers.add_parser('baypass',
                                           help="Process Baypass results")
    lfmm_parser = subparsers.add_parser('lfmm',
                                        help="Process LFMM results")

    # Use a loop to add common options to both subparsers. It is hackish,
    # but seems to work better than the "inherits" alternative.
    for subparser in [baypass_parser, lfmm_parser]:
        # Group definitons
        io_opts = subparser.add_argument_group("Input/Output options")
        parameters = subparser.add_argument_group("Program execution options")
        misc_opts = subparser.add_argument_group("Miscellaneous options")

        # Group options
        parameters.add_argument("-covars", dest="num_covars", type=int,
                                default=3, required=False,
                                help="Number of covars to calculate the RONA "
                                "for. Default is 3.")

        parameters.add_argument("-remove-outliers", dest="outliers",
                                action='store_true',
                                default=False, required=False,
                                help="Set this option to remove outliers "
                                     "from the RONA ananlysis. This is "
                                     "not really recommended and is here "
                                     "for historical purposes.")

        parameters.add_argument("-immutables", dest="immutables",
                                default=[], required=False,
                                nargs="+",
                                help="List of immutable covariates. These "
                                     "are not even parsed from the betai "
                                     "or p-values file. By default no covars "
                                     "are skipped. You can enter any number of "
                                     "column numbers, separated by spaces. "
                                     "eg.:`-immutables 1 2 3` to skip columns "
                                     "1, 2 and 3.")

        parameters.add_argument("-ronatype", dest="rtype", type=str,
                                default="absdiff", required=False,
                                choices=["diff", "absdiff", "dist"],
                                help="Type of RONA to calculate. Default is "
                                     "absolute difference as in Rellstab et "
                                     "al. 2016. Other options are "
                                     "'difference' (not absolute) and "
                                     "'distance' (future vs. trendline).")

        io_opts.add_argument("-pc", dest="present_covars_file", type=str,
                             required=True, help="File with Present "
                                                 "environmental data.")

        io_opts.add_argument("-fc", dest="future_covars_file", type=str,
                             required=True, help="File with Future "
                                                 "environmental data.")

        io_opts.add_argument("-out", dest="outfile", type=str,
                             required=True, help="Path to where RONA plot "
                                                 "should be saved. Supports "
                                                 "PDF, SVG and PNG "
                                                 "extensions.")

        io_opts.add_argument("-map", dest="map_filename", type=str,
                             required=False, default=None,
                             help="Provide a path here if you want to draw "
                             "a map plot of your samples. The extension will "
                             "determine the file type (png, svg or pdf).")

        misc_opts.add_argument("-draw-ind-plots", dest="plots",
                               default=None, required=False, type=str,
                               help="Path to where individual plots should be "
                                    "saved. Omitting this parameter will "
                                    "cause individual plots not to be drawn.")

        misc_opts.add_argument("-no-weighted-means", dest="use_weights",
                               action='store_false',
                               help="Pass this option if you don't want to "
                                    "use weighted means for RONA "
                                    "calculations.",
                               required=False, default=True)

    # Baypass Group
    # Group definitions
    baypass_io_opts = baypass_parser.add_argument_group("Input/Output "
                                                        "options")
    baypass_parameters = baypass_parser.add_argument_group("Program execution"
                                                           " options")

    # Group options
    baypass_parameters.add_argument("-bf", dest="bayes_factor", type=float,
                                    required=True,
                                    help="Bayes factor treshold for "
                                    "considering associations.")

    baypass_io_opts.add_argument("-pop", dest="popnames_file", type=str,
                                 required=True,
                                 help="File with population names.")

    baypass_io_opts.add_argument("-beta", dest="baypass_summary_betai_file",
                                 type=str, required=True,
                                 help="Baypass summary betai file.")

    baypass_io_opts.add_argument("-pij", dest="baypass_pij_file", type=str,
                                 required=True, help="Baypass pij file.")

    # LFMM Group
    # Group definitions
    lfmm_io_opts = lfmm_parser.add_argument_group("Input/Output "
                                                  "options")
    lfmm_parameters = lfmm_parser.add_argument_group("Program execution"
                                                     " options")

    # Group options
    lfmm_parameters.add_argument("-P", dest="p_thres", type=float,
                                 required=True,
                                 help="p-value treshold for "
                                 "considering associations.")

    lfmm_io_opts.add_argument("-assoc", dest="lfmm_assoc_file",
                              type=str, required=True,
                              help="LFMM associations file.")

    lfmm_io_opts.add_argument("-geno", dest="allele_freqs_file", type=str,
                              required=True, help="LFMM input file.")

    arguments = parser.parse_args(args)

    return arguments
